#!/usr/bin/perl
#
# Flowlabel test programs.
#
# Compile:
# term> ./make.pl [<options>]
#   <options>
#	clean :		remove exec files.
#	tcp :		make exec files using tcp socket.
#	timer :		timing check for linger and expires counters.
#	test [<exp>]:	test for flowlabel cleaning and expires timer.
#	getaddrinfo :	make exec files using getaddrinfo function.
#	newheader :	copy /usr/include/linux/{in6,ipv6}.h and apply patches.
#
# Execute:
# ex) UDP version
# term1> ./server 1234 localhost 10000
# term2> ./client 9876 localhost 10000
#
# (result)
# term1> ./server 1234 localhost 10000
# bind_addr = ::1
# binding...
# level = 41, type = 11
# dst_host flow has 0x9876.
# dst_port: 41173			// port number is decided randomly.
# dst_addr = ::1
#
# term2> ./client 9876 localhost 10000
# dst_addr = ::1
# sending...				// and wait more than 1 second.
# src_addr = ::
# level = 41, type = 11
# dst_host flow has 0x1234.
# message: flowlabel = 0x1234
#
#
#$SEL = "";			// Use getaddrinfo() to get IPv6 address.
# (But it returns wrong address(res->ai_addr) on SuSE 8.0FTP + usagi-cvs.)
#

$CC = "gcc -Wall";
$USAGI = "-L/usr/local/v6/lib -linet6";
$INC = "-Iinclude";
$SEL = "-DSELECT_HBN2";
$USRINC = "/usr/include";

$SERVER = "server_udp.c";
$CLIENT = "client_udp.c";

while(@ARGV) {
	$_ = shift;
#	print "$_\n";

	if (/^clean/) {
		system("rm -f client server *~");
		print "Done.\n";
		exit(0);
	} elsif (/^newheader/) {
		chdir "include/linux/";
		rename "in6.h", "in6.h.old";
		rename "ipv6.h", "ipv6.h.old";
		system("cp $USRINC/linux/in6.h .; cp $USRINC/linux/ipv6.h .");
		system("patch < in6.h.diff; patch < ipv6.h.diff");
		print "Done. Old headers ware renamed to *.old.\n";
		exit(0);
	} elsif (/^tcp/) {
		$SERVER = "server_tcp.c";
		$CLIENT = "client_tcp.c";
	} elsif (/^pktinfo/) {
		$SEL = "$SEL -DUSE_PKTINF";
	} elsif (/^getaddrinfo/) {
		$SEL = "";
	} elsif (/^timer/) {
		print "linger and expires counter check\n";
		$SERVER = "server_timer.c";
		$CLIENT = "client_timer.c";
	} elsif (/^test/) {
		print "label cleaning test\n";
		$_ = shift;
		if (/^[0123456789]/) {
			$SEL = "$SEL -DEXPTIMER=$_";
		}
		$SERVER = "server_test.c";
		$CLIENT = "client_test.c";
	} elsif (/^-I/) {
		$INC = "$INC $_";
	} elsif (/^nousagi/) {
		$USAGI = "";
	} elsif (/^noinc/) {
		$INC = "";
	} else {
		print "Invalid option: $_\n";
	}
}

system("$CC $SEL $INC $USAGI -o client $CLIENT");
system("$CC $SEL $INC $USAGI -o server $SERVER");

